/***************************************************************************
 *   Copyright (C) 2005 by JSRiera					   *
 *   http://www.xtec.cat/dnee/udc/english/main.html                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 *   YOU CAN NOT USE THIS SOURCE FILE OR MODIFIED VERSION FOR ANY          *
 *   COMMERCIAL PRODUCTS!                                                  *
 *                                                                         *
 ***************************************************************************/

#include "CImgProc.h"



/**********************************************************************
*  GENERAL															  *
***********************************************************************/


CImgProc::CImgProc()
{

	// general
	capture = 0; frame = 0; image = 0;	
	bppixel = 3;
	
	// color
	hsv = 0; hue = 0; 
    soroll = 3; 

	for (int i=0; i<MAX_COLORS;i++)
	{
		mask[i] = 0;
		distmin[i] = 10000;
		hist[i] = 0;
		cbackprj[i] = 0;
		histimg[i] = 0;
		cstorage[i] = 0;
		find[i] = 0;
		imagecp[i] = 0;
	}

	// move
	grey = 0; pre_image = 0; 
	add_abs_image = 0; 
	pre_grey = 0;
	abs_image = 0; abs_images = 0;
	punts_no_zero = 0;
	mstorage = 0; contour = 0;
	mthreshold = 100;


}

//CImgProc::~CImgProc ()
//{
//	cvReleaseCapture( &capture );
//}

void CImgProc::FreeWebcam ()
{
	//fclose(fp);	
	if (mode == 0 )
		cvReleaseCapture( &capture );
/*
	if (mode == 1)
	{
		arVideoCapStop();
		arVideoClose();
	}
*/
}

int CImgProc::InitializeWebcam (int m_mode, int f_vertical, int f_horizontal)
{
	mode = m_mode;
	f_vert = f_vertical;
	f_horz = f_horizontal;

	//fp=fopen("temp.txt", "w"); 


	if ( mode == 0 )
	{
		capture = cvCaptureFromCAM(-1);
		if (!capture) {
			printf("Could not initialize WebCam\n");
			return -1;
		}
		else return 0;
	}
	return -1;
}

int CImgProc::CaptureFrame (int width, int height)
{
	if ( mode == 0 )
	{
		frame = cvQueryFrame( capture );
		if (!frame) return 0;
		else {
			if (f_vert)
				cvFlip(frame, frame, 0);
			if (f_horz)
				cvFlip(frame, frame, 1);
			xsize = frame->width;
			ysize = frame->height;
		}

		if ((width != 0) && (height != 0)) // values different zero, resize image
		{
			if (!image)
				image = cvCreateImage( cvSize(width,height), 8, 3);
			cvResize( frame, image, 0);
		}
		else // one of those values are zero, we no resize image
		{
			if (!image)
				image = cvCreateImage( cvGetSize(frame), 8, 3);
			cvCopy( frame, image, 0 );
		}
	}

	return 1;
}

void CImgProc::GetFrameSize (int *width, int *height)
{
	*width = xsize;
	*height = ysize;
}


IplImage* CImgProc::GetPicture (int which, int index)
{
	if ((which == IMAGE) && (image != 0))
	{
		return image;
	}
	else if ((which == COLOR_BKG) && (cbackprj[index] != 0))
	{
		//cvMerge(cbackprj[index], cbackprj[index], cbackprj[index], 0, cbackprj4[index]);
		return cbackprj4[index];
	}
	else if ((which == MOVE_BKG) && (mbackprj != 0))
	{
		cvMerge(mbackprj, mbackprj, mbackprj, 0, mbackprj4);
		return mbackprj4;
	}
	else if ((which == HGEN_IMG) && (histgen != 0))
	{
		return histgen;
	}
	else if ((which == HCOL_IMG) && (histimg[index] != 0))
	{
		return histimg[index];
	}
	else
		return 0;

}

// Guarda la IplImage Image en un fitxer
void CImgProc::SaveImageToFile (const char *filename, int which)
{
	if (which == IMAGE)
		cvSaveImage(filename, image);
	if (which == COLOR_BKG)
	{
		cvCvtColor( cbackprj[0], cbackprj4[0], CV_GRAY2BGR );
		cvSaveImage(filename, cbackprj4[0]);
	}
}

// Inverteix la imatge hortzontalment o verticalment o les dues 
// coses
void CImgProc::ManipulateImage (int mode)
{
	if (mode == 0x00000001) {
		cvFlip(image, image, 0);
	}
	if (mode == 0x00000010) {
		cvFlip (image, image, 1);
	}
	if (mode == 0x00000011) {
		cvFlip (image, image, -1);
	}
}

void CImgProc::ConvertBGRToRGB(IplImage * image)
{
	char color;
		
	for (int i=0; i < image->imageSize; i+=bppixel) {
			
		color                 = image->imageData[i];
		image->imageData[i]   = image->imageData[i+2];
		image->imageData[i+2] = color;
	}					
}

void CImgProc::SourceDlgProperties ()
{
//    cvSetCaptureProperty(capture, CV_CAP_PROP_DIALOG_SOURCE, 0);
}

void CImgProc::FormatDlgProperties ()
{
//    cvSetCaptureProperty(capture, CV_CAP_PROP_DIALOG_FORMAT, 0);
}

void CImgProc::ResetCapture ()
{
    image = NULL;
	hsv = NULL;
	pre_image = NULL;
	for (int i = 0; i<MAX_COLORS; i++)
		cbackprj[i] = NULL;
}

/**********************************************************************
*  COLOR TRACKING													  *
***********************************************************************/

// Crea i inicialitza les variables necessries per a poder
// usar les rutines de seguiment
void CImgProc::InitializeColorTracking (int index)
{
    int hdims = 16;     // 48
	float hranges_arr[] = {0,180};
	float* hranges = hranges_arr;

    if (!hsv) {
	    hsv = cvCreateImage( cvGetSize(image), 8, bppixel );
		hue = cvCreateImage( cvGetSize(image), 8, 1 );
		histgen = cvCreateImage (cvSize (320, 200), 8, bppixel);
		hist_foto = cvCreateHist( 1, &hdims, CV_HIST_ARRAY, &hranges, 1 );  
		bprjcp = cvCreateImage( cvGetSize(image), 8, 1 );
		//hist[index] = cvCreateImage( cvSize(320,200), 8, bppixel );
		//ObtenirHistFitxer(fitxer);
		//cvZero( histimg );
	    //}
    }
	if (!cbackprj[index])
	{
		imagecp[index] = cvCreateImage( cvGetSize(image), 8, 3);
	    mask[index] = cvCreateImage( cvGetSize(image), 8, 1 );
		cbackprj[index] = cvCreateImage( cvGetSize(image), 8, 1 );
		//bprjcp = cvCreateImage( cvGetSize(image), 8, 1 );
		cbackprj4[index] = cvCreateImage( cvGetSize(image), 8, bppixel );
		hist[index] = cvCreateHist( 1, &hdims, CV_HIST_ARRAY, &hranges, 1 );  
		histimg[index] = cvCreateImage( cvSize(320,200), 8, bppixel );
		cstorage[index] = cvCreateMemStorage(0);
	}
}

// Calculates color background. Color is indicated with the file = file
// and returns the index associated to that color
int CImgProc::CalculateColorBackground (char *file, int num_color, int maxx, int maxy,
										int minx, int miny, int vmin, int smin)
{
	int				px, py;
	int				diff_x, diff_y;				// Distance with the cur and prev point detected
	int				x_ant, y_ant;				// Previous point detected
	CvRect			rect;						// Rectangle (on detected marker)
	CvSeq			*contours;					// Markers detected
	int				distancia;					// Distance


	if ((num_color < 0) || (num_color > MAX_COLORS)) return -1;

	InitializeColorTracking(num_color); 

    GetHistFromFile(file, num_color);                  
   	//GetHistParams (num_color, &add, &sub, &vmin, &smin);
	//add = 0; sub = 0; vmin = 20; smin = 10;
	//cvCopy (image, imagecp[num_color], 0);
	//cvAddS (imagecp[num_color], cvScalar(add, add, add), imagecp[num_color], NULL);
	//cvSubS (imagecp[num_color], cvScalar(sub, sub, sub), imagecp[num_color], NULL);

    //cvCvtColor( imagecp[num_color], hsv, CV_BGR2HSV );  // BGR to HSV
	cvCvtColor( image, hsv, CV_BGR2HSV );  // BGR to HSV
	cvInRangeS( hsv, cvScalar(0,smin,MIN(vmin,256),0),
    cvScalar(180,256,MAX(vmin,256),0), mask[num_color] );  // MASK
    cvSplit(hsv, hue, 0, 0, 0);
    cvCalcBackProject(&hue, cbackprj[num_color], hist[num_color]);
    cvCalcHist(&hue, hist_foto, 0, mask[num_color]);

    cvZero(histgen);       	
    histimg[num_color] = HistogramToIplImage(hist[num_color], histimg[num_color], 16);
    histgen = HistogramToIplImage(hist_foto, histgen, 16);

    cvAnd(cbackprj[num_color], mask[num_color], cbackprj[num_color], 0);

	find[num_color] = 0;
	//rect.x = 0; rect.y = 0; rect.width = 0; rect.height = 0;
	px = 0;
	py = 0;

	if (cbackprj[num_color] == NULL) return -1;

   	cvCopy( cbackprj[num_color], bprjcp, 0 );

	cvFindContours( bprjcp, cstorage[num_color], &contours, sizeof(CvContour),
			CV_RETR_LIST, CV_CHAIN_APPROX_NONE, cvPoint(0,0));

    for( ; contours != 0; contours = contours->h_next )
    {
		// Filter contours detected
		CvRect tmp = cvBoundingRect(contours, 0);
		if ((tmp.width > minx) && (tmp.height > miny)  
			&& (tmp.width < maxx) && (tmp.height < maxy)) {


			// distance within the current point and the previous point
			distancia = (int) sqrt(pow(abs(tmp.x - px), 2) + pow(abs(tmp.y - py), 2));
			if (distancia < distmin[num_color]) {
				// Actualize values
				rect = tmp;
				distmin[num_color] = distancia;
				px = rect.x;
				py = rect.y;

				// control if the color detected is valid
				find[num_color] = 1;

				// Filter noise: this calculate the differences between points
				// if the difference is higher than determinate threshold
				// we actualize the point, if not the point romain static
				diff_x = abs(px - x_ant);
				diff_y = abs(py - y_ant);
				if (diff_x > soroll) x_ant = px;
				px = x_ant;
				if (diff_y > soroll) y_ant = py;
				py = y_ant;

				puntx[num_color] = px;
				punty[num_color] = py;
			}
		}
	}

	cvClearMemStorage(cstorage[num_color]);

	distmin[num_color] = 10000;
    cvRectangle(image, cvPoint(rect.x, rect.y), cvPoint(rect.x+rect.width, rect.y+rect.height), CV_RGB(255, 0, 0), 3, 8, 0); 
	cvCvtColor( cbackprj[num_color], cbackprj4[num_color], CV_GRAY2BGR );
    cvRectangle(cbackprj4[num_color], cvPoint(rect.x, rect.y), cvPoint(rect.x+rect.width, rect.y+rect.height), CV_RGB(255, 0, 0), 3, 8, 0); 

	return 0;
}


// Return the position of the color of associated index
int CImgProc::GetColorPosition (int index, int *x, int *y)
{

	if ((index <0) || (index > MAX_COLORS - 1)) return -1;
	*x = puntx[index];
	
	*y = punty[index];

	if (find[index] == 0) return 0;
	else return 1;

}

int CImgProc::SaveParameters (char *file, int index, int maxx, int maxy, int minx, int miny,
										int vmin, int smin)
{
	char buffer[255];
	FILE *fp;

	if((fp=fopen(file, "r+")) == NULL)
	{
		printf("ERROR opening file %s\n\n", file);
		return -1;
	}

	if (index == 0)
	{
		sprintf(buffer, "%d %d %d %d %d %d", maxx, maxy, minx, miny, vmin, smin);
		fputs (buffer, fp);
	}
	if (index == 1)
	{
		fgets(buffer, 255, fp);  
		sprintf(buffer, "%d %d %d %d %d %d", maxx, maxy, minx, miny, vmin, smin);
		fputs (buffer, fp);
	}
	if (index == 2)
	{
		fgets(buffer, 255, fp);  
		fgets(buffer, 255, fp);  
		sprintf(buffer, "%d %d %d %d %d %d", maxx, maxy, minx, miny, vmin, smin);
		fputs (buffer, fp);
	}
	if (index == 3)
	{
		fgets(buffer, 255, fp);  
		fgets(buffer, 255, fp);  
		fgets(buffer, 255, fp);  
		sprintf(buffer, "%d %d %d %d %d %d", maxx, maxy, minx, miny, vmin, smin);
		fputs (buffer, fp);
	}

	fclose(fp);	
	return 0;

}

int CImgProc::GetParameters (char *file, int index, int *maxx, int *maxy, int *minx, int *miny,
										int *vmin, int *smin)
{
	char buffer[255];
	FILE *fp;
	int a, b, c, d, e, f;

	if((fp=fopen(file, "r")) == NULL)
	{
		printf("ERROR opening file %s\n\n", file);
		return -1;
	}


	if (index == 0)
	{
		fgets(buffer, 255, fp);  
		sscanf(buffer, "%d %d %d %d %d %d", &a, &b, &c, &d, &e, &f);  
	}
	if (index == 1)
	{
		fgets(buffer, 255, fp);  
		fgets(buffer, 255, fp);  
		sscanf(buffer, "%d %d %d %d %d %d", &a, &b, &c, &d, &e, &f);  
	}
	if (index == 2)
	{
		fgets(buffer, 255, fp);  
		fgets(buffer, 255, fp);  
		fgets(buffer, 255, fp);  
		sscanf(buffer, "%d %d %d %d %d %d", &a, &b, &c, &d, &e, &f);  
	}
	if (index == 3)
	{
		fgets(buffer, 255, fp);  
		fgets(buffer, 255, fp);  
		fgets(buffer, 255, fp);  
		fgets(buffer, 255, fp);  
		sscanf(buffer, "%d %d %d %d %d %d", &a, &b, &c, &d, &e, &f);  
	}

	fclose(fp);	

	*maxx = a; *maxy = b;
	*minx = c; *miny = d;
	*vmin = e; *smin = f;

	return 0;

}


void CImgProc::SetNoiseThreshold (int noise)
{
	soroll = noise;
}

void CImgProc::GetHistFromFile (const char *file, int index)
{
	hist[index]->bins = (CvHistogram *)cvLoad(file);
}

// escribim xml a ma, sino en Linux no funciona correctament
// hi ha un problema amb els decimals
void CImgProc::SaveHistFile (const char *file, int index)
{
	FILE *fc;
	char num[20];

   	if ((fc=fopen(file, "w")) == NULL)
	{
		printf("ERROR salvant fitxer %s", file);
		exit(1);
	}
    fputs("<?xml version=\"1.0\"?>\n", fc);
    fputs("<opencv_storage>\n", fc);
    fputs("<hist type_id=\"opencv-nd-matrix\">\n", fc);
    fputs("<sizes>\n", fc);
    fputs("16</sizes>\n", fc);
    fputs("<dt>f</dt>\n", fc);
    fputs("<data>\n", fc);

    for (int i=0; i<16; i++)
    {
        float *val;
        int ival;
        val = cvGetHistValue_1D(hist[index], i);
        ival = (int)floor(*val);
        if (ival < 0) fputs("0\n", fc);
        else {
             if (ival < 20) fputs("0", fc);
             else fputs("400", fc);
             fputs("\n", fc);
        } 
        //_itoa( ival, num, 10 );
        //fputs(num, fc);
		//fputs ("\n", fc);
    }
    fputs("</data></hist>\n", fc);
    fputs("</opencv_storage>", fc);
	fclose (fc);
}

CvScalar CImgProc::hsv2rgb( float hue )
{
    int rgb[3], p, sector;
    static const int sector_data[][3]=
        {{0,2,1}, {1,2,0}, {1,0,2}, {2,0,1}, {2,1,0}, {0,1,2}};
    hue *= 0.033333333333333333333333333333333f;
    sector = cvFloor(hue);
    p = cvRound(255*(hue - sector));
    p ^= sector & 1 ? 255 : 0;

    rgb[sector_data[sector][0]] = 255;
    rgb[sector_data[sector][1]] = 0;
    rgb[sector_data[sector][2]] = p;

    return cvScalar(rgb[2], rgb[1], rgb[0],0);
}

IplImage* CImgProc::HistogramToIplImage(CvHistogram* hist, IplImage* histimg, int hdims) 
{
	int bin_w, i;

	bin_w = histimg->width / hdims;  
	
    for( i = 0; i < hdims; i++ )
    {
		int val = cvRound( cvGetReal1D(hist->bins,i)*histimg->height/255 );
        CvScalar color = hsv2rgb(i*180.f/hdims);
        cvRectangle( histimg, cvPoint(i*bin_w,histimg->height),
        cvPoint((i+1)*bin_w,histimg->height - val), color, -1, 8, 0 );
    }

	return histimg;
}

int CImgProc::CreateHistogram(int x, int y, int index)
{
	int hdims = 16;

	if ((x >= frame->width) || (y >= frame->height)) return -1;
    else 
    {
    	cvZero(histimg[index]);
	    CvRect sel = {x - 10, image->height - y - 10, 20, 20};
    //cvRectangle(image, cvPoint(sel.x, sel.y), cvPoint(sel.x+sel.width, sel.y+sel.height), CV_RGB(0, 255, 0), 1, 8, 0); 
     	int vmin = 10;
      	int vmax = 256;
       	int smin = 30;
       	float hranges_arr[] = {0, 180};
       	float* hranges = hranges_arr;
        IplImage * hv = cvCreateImage( cvGetSize(frame), 8, 3 );
        IplImage * he = cvCreateImage( cvGetSize(frame), 8, 1 );
        IplImage * mk = cvCreateImage( cvGetSize(frame), 8, 1 );
       	CvHistogram * ht = cvCreateHist( 1, &hdims, CV_HIST_ARRAY, &hranges, 1 );  

        cvCvtColor( frame, hv, CV_BGR2HSV );  
        cvInRangeS( hv, cvScalar(0,smin,MIN(vmin,vmax),0),
        cvScalar(180,256,MAX(vmin,vmax),0), mk ); 
       	cvSplit(hv, he, 0, 0, 0);
       	cvSetImageROI( he, sel);
       	cvSetImageROI( mk, sel);
       	cvCalcHist(&he, hist[index], 0, mk);
       	cvResetImageROI( he );  
       	cvResetImageROI( mk );

        cvReleaseImage( &hv);
       	cvReleaseImage( &he );
       	cvReleaseImage( &mk );
       	cvReleaseHist ( &ht );
        return 0;
    }
}

void CImgProc::WriteNumbers (int punts)
{
     CvFont font;
 	 cvInitFont(&font, CV_FONT_HERSHEY_PLAIN, 4, 4, 0, 5, 0);
// 	 if ( !horz ) {
 	    if (punts == 0) {
           cvPutText(image, "I", cvPoint(40, image->height-60), &font, CV_RGB(0, 0, 255));
           cvPutText(image, "II", cvPoint(image->width-60, image->height-60), &font, CV_RGB(0, 0, 255));
           cvPutText(image, "III", cvPoint(image->width-60, 40), &font, CV_RGB(0, 0, 255));
           cvPutText(image, "IIII", cvPoint(40, 40), &font, CV_RGB(0, 0, 255));
        }
 	    if (punts == 1) {
           cvPutText(image, "I", cvPoint(40, image->height-60), &font, CV_RGB(0, 255, 0));
           cvPutText(image, "II", cvPoint(image->width-60, image->height-60), &font, CV_RGB(0, 0, 255));
           cvPutText(image, "III", cvPoint(image->width-60, 40), &font, CV_RGB(0, 0, 255));
           cvPutText(image, "IIII", cvPoint(40, 40), &font, CV_RGB(0, 0, 255));
        }
 	    if (punts == 2) {
           cvPutText(image, "I", cvPoint(40, image->height-60), &font, CV_RGB(0, 255, 0));
           cvPutText(image, "II", cvPoint(image->width-60, image->height-60), &font, CV_RGB(0, 255, 0));
           cvPutText(image, "III", cvPoint(image->width-60, 40), &font, CV_RGB(0, 0, 255));
           cvPutText(image, "IIII", cvPoint(40, 40), &font, CV_RGB(0, 0, 255));
        }
 	    if (punts == 3) {
           cvPutText(image, "I", cvPoint(40, image->height-60), &font, CV_RGB(0, 255, 0));
           cvPutText(image, "II", cvPoint(image->width-60, image->height-60), &font, CV_RGB(0, 255, 0));
           cvPutText(image, "III", cvPoint(image->width-60, 40), &font, CV_RGB(0, 255, 0));
           cvPutText(image, "IIII", cvPoint(40, 40), &font, CV_RGB(0, 0, 255));
        }
        if (punts == 4) {
           cvPutText(image, "I", cvPoint(40, image->height-60), &font, CV_RGB(0, 255, 0));
           cvPutText(image, "II", cvPoint(image->width-60, image->height-60), &font, CV_RGB(0, 255, 0));
           cvPutText(image, "III", cvPoint(image->width-60, 40), &font, CV_RGB(0, 255, 0));
           cvPutText(image, "IIII", cvPoint(40, 40), &font, CV_RGB(0, 255, 0));
        }
/*      } else {
 	    if (punts == 0) {
           cvPutText(image, "II", cvPoint(40, image->height-60), &font, CV_RGB(0, 0, 255));
           cvPutText(image, "I", cvPoint(image->width-60, image->height-60), &font, CV_RGB(0, 0, 255));
           cvPutText(image, "IIII", cvPoint(image->width-60, 40), &font, CV_RGB(0, 0, 255));
           cvPutText(image, "III", cvPoint(40, 40), &font, CV_RGB(0, 0, 255));
        }
 	    if (punts == 1) {
           cvPutText(image, "II", cvPoint(40, image->height-60), &font, CV_RGB(0, 255, 0));
           cvPutText(image, "I", cvPoint(image->width-60, image->height-60), &font, CV_RGB(0, 0, 255));
           cvPutText(image, "IIII", cvPoint(image->width-60, 40), &font, CV_RGB(0, 0, 255));
           cvPutText(image, "III", cvPoint(40, 40), &font, CV_RGB(0, 0, 255));
        }
 	    if (punts == 2) {
           cvPutText(image, "II", cvPoint(40, image->height-60), &font, CV_RGB(0, 255, 0));
           cvPutText(image, "I", cvPoint(image->width-60, image->height-60), &font, CV_RGB(0, 255, 0));
           cvPutText(image, "IIII", cvPoint(image->width-60, 40), &font, CV_RGB(0, 0, 255));
           cvPutText(image, "III", cvPoint(40, 40), &font, CV_RGB(0, 0, 255));
        }
 	    if (punts == 3) {
           cvPutText(image, "II", cvPoint(40, image->height-60), &font, CV_RGB(0, 255, 0));
           cvPutText(image, "I", cvPoint(image->width-60, image->height-60), &font, CV_RGB(0, 255, 0));
           cvPutText(image, "IIII", cvPoint(image->width-60, 40), &font, CV_RGB(0, 255, 0));
           cvPutText(image, "III", cvPoint(40, 40), &font, CV_RGB(0, 0, 255));
        }
        if (punts == 4) {
           cvPutText(image, "II", cvPoint(40, image->height-60), &font, CV_RGB(0, 255, 0));
           cvPutText(image, "I", cvPoint(image->width-60, image->height-60), &font, CV_RGB(0, 255, 0));
           cvPutText(image, "IIII", cvPoint(image->width-60, 40), &font, CV_RGB(0, 255, 0));
           cvPutText(image, "III", cvPoint(40, 40), &font, CV_RGB(0, 255, 0));
        }
      }*/
}

// per dibuixar els punts en mode tauleta
void CImgProc::WritePoints (int p1x, int p1y, int p2x, int p2y, int p3x,
                                int p3y, int p4x, int p4y)
{
   cvRectangle (image, cvPoint(p1x, p1y), cvPoint(p1x+5, p1y+5), CV_RGB(0,255,0), CV_FILLED, 8, 0);   
   cvRectangle (image, cvPoint(p2x, p2y), cvPoint(p2x+5, p2y+5), CV_RGB(0,255,0), CV_FILLED, 8, 0);   
   cvRectangle (image, cvPoint(p3x, p3y), cvPoint(p3x+5, p3y+5), CV_RGB(0,255,0), CV_FILLED, 8, 0);   
   cvRectangle (image, cvPoint(p4x, p4y), cvPoint(p4x+5, p4y+5), CV_RGB(0,255,0), CV_FILLED, 8, 0);   
}

void CImgProc::DrawCircle (int x, int y)
{
     cvCircle(image, cvPoint(x, y), 5, CV_RGB(255,0,0), CV_FILLED, 8, 0);  
}

// escriure linies en mode Joystick
void CImgProc::PaintLines(int bh, int bv, int bh1, int bh2, int bv1, int bv2)
{
     if ( bh == 1 )
          cvLine(image, cvPoint(0,bh1), cvPoint(image->width,bh1), CV_RGB(255,0,0), 1, 8, 0);
     if ( bh == 2 ) {          
          cvLine(image, cvPoint(0,bh1), cvPoint(image->width,bh1), CV_RGB(255,0,0), 1, 8, 0);
          cvLine(image, cvPoint(0,bh2), cvPoint(image->width,bh2), CV_RGB(255,0,0), 1, 8, 0);
     }
     if ( bv == 1 )
          cvLine(image, cvPoint(bv1,0), cvPoint(bv1,image->height), CV_RGB(255,0,0), 1, 8, 0);
     if ( bv == 2 ) {          
          cvLine(image, cvPoint(bv1,0), cvPoint(bv1,image->height), CV_RGB(255,0,0), 1, 8, 0);
          cvLine(image, cvPoint(bv2,0), cvPoint(bv2,image->height), CV_RGB(255,0,0), 1, 8, 0);
     }
}


/**********************************************************************
*  MOVE TRACKING													  *
***********************************************************************/

// Inicialitza i crea les variables per poder utilitzar les
// rutines del moviment
int CImgProc::InitializeMoveTracking ()
{
	if (!pre_image ) {
	    //frame = cvCreateImage( cvSize(xsize, ysize), 8, 3 );
		pre_image = cvCreateImage( cvGetSize(image), 8, bppixel );       
		abs_image = cvCreateImage( cvGetSize(image), 8, 1 );
		add_abs_image = cvCreateImage( cvGetSize(image), 8, 1 );
   
	    grey = cvCreateImage( cvGetSize(image), 8, 1 );
		pre_grey = cvCreateImage( cvGetSize(image), 8, 1 );

		mbackprj = cvCreateImage( cvGetSize(image), 8, 1 );
		abs_image->origin = image->origin;
		add_abs_image->origin = image->origin;
		mbackprj->origin = image->origin;
		mstorage = cvCreateMemStorage(0);

		mbackprj4 = cvCreateImage(cvGetSize(image),8,bppixel);
		mbackprj4->origin = image->origin;

		abs_images = cvCreateImage( cvGetSize(image), 8, 1 );
		cvZero(abs_images);
		return 1;
	}
	return 0;
}

// Obt una imatge en blanc i negre del moviment
int CImgProc::CalculateMovement ()
{

	InitializeMoveTracking ();
    //cvCopy( frame, image, 0 );
	//cvSetImageData( frame, dataPtr, xsize * 3);
	//cvMirror(image, NULL, 0);

	cvCvtColor(image, grey, CV_BGR2GRAY);
	cvZero(add_abs_image);
	cvCvtColor(pre_image, pre_grey, CV_BGR2GRAY);
 
	cvAbsDiff( grey,pre_grey, abs_image );

    cvThreshold( abs_image, abs_image, mthreshold, 255, CV_THRESH_BINARY );

	cvAdd(abs_images,add_abs_image,add_abs_image);
	cvThreshold( add_abs_image, add_abs_image, 20, 255, CV_THRESH_BINARY );

    cvFindContours( abs_image, mstorage, &contour, sizeof(CvContour), CV_RETR_CCOMP, CV_CHAIN_APPROX_SIMPLE );
    cvZero( mbackprj );


    for( ; contour != 0; contour = contour->h_next )
    {
		CvScalar color = CV_RGB( 255, 255, 255 );
        // replace CV_FILLED with 1 to see the outlines    
  
        cvDrawContours( mbackprj, contour, color, color, -1, 6, 0 );
    }
	// sempre invertim fons, perque alguna funci d'amunt ens la inverteix
	// cvFlip (mbackprj, mbackprj, 0);
	cvCopy(abs_images,abs_image,0);
	cvCopy(image,pre_image,0);

	return 0;
}

// Change the value defined for the move threshold to reduce
// noise. Default = 100
int CImgProc::SetMoveThreshold (int thres)
{
	mthreshold = thres;
	return 0;
}

// Retorna el nombre de punts blancs de la regi donada.
int CImgProc::GetPixelsNoZero (int x, int y, int w, int h)
{
	if ((x < 0) || (y < 0) || (w < 0) || (h < 0)
		|| (w > 640) || (h > 480) || (x > 640) || (y > 480)) return 0;

	CvRect rect = {x, y, w, h};
//	CvRect r_f;
//	double total;
	if (mbackprj) {
		// atenci, treure flip en alguna funci de fora
		cvSetImageROI(mbackprj, rect);
		punts_no_zero = cvCountNonZero(mbackprj);
		cvResetImageROI(mbackprj);

	}
	return punts_no_zero;
}


 
 
 
 
 
 
 
 
 
 
 
 
 
