/***************************************************************************
 *   Copyright (C) 2005 by JSRiera					   *
 *   http://www.xtec.cat/dnee/udc/english/main.html                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 *   YOU CAN NOT USE THIS SOURCE FILE OR MODIFIED VERSION FOR ANY          *
 *   COMMERCIAL PRODUCTS!                                                  *
 *                                                                         *
 ***************************************************************************/
#ifndef CIMGPROC_H
#define CIMGPROC_H

#include <cv.h>
#include <highgui.h>
#include <stdio.h>

#define  VERTICAL	  0x00000001
#define  HORITZONTAL  0x00000010
#define  IMAGE		  0
#define  COLOR_BKG    1
#define  MOVE_BKG     2
#define  HGEN_IMG	  3	
#define  HCOL_IMG     4
#define  MAX_COLORS   4
#define WIDTHBYTES(bits) ((((bits) + 31) / 32) * 4)  


class CImgProc
{

public:

	/****************************************************
	*
	*  GENERAL
	*
	*****************************************************/

	// Constructor
	CImgProc();

	// Destructor
	//~CImgProc();

	// Frees webcam to be able to use in other applications
	void FreeWebcam ();

	// m_mode == 0 => OpenCV (VFW)
	// m_mode == 1 => ARvideo (DirectShow)
	// Return -1 if cannot initialize, 0 otherwise
	int InitializeWebcam (int m_mode, int f_vertical, int f_horizontal);

	// Capture a frame of the webcam
	// It must be called inside a bucle to obtain a
	// sequence of images.
	// Return -1 on error, 0 otherwise
	int CaptureFrame (int width, int height);

	// Retrieve the size of original frame
	void GetFrameSize (int *width, int *height);

	// Retrieve the image indicated. Index only necessary in color mode
	IplImage* GetPicture (int which, int index);

	// Save captured image in a file
	void  SaveImageToFile (const char *filename, int which);

	// Inverts the image, horizontal, vertical or both
	void  ManipulateImage (int mode);

	// Swaps the B and G components	
	void ConvertBGRToRGB(IplImage * image);

	// Open a window to select different properties of webcam. Only in  
	// windows and with VFW drivers.
	void SourceDlgProperties ();

	// Open a window to change the image size. Only in windows and
	// with VFW drivers.
	void FormatDlgProperties ();

	// Reinitializes image
	void ResetCapture ();


	/****************************************************
	*
	*  COLOR TRACKING
	*
	*****************************************************/

	// Return 1 if the color is found, 0 otherwise
	// x, y are the position of the color inside image
	int  GetColorPosition (int index, int *x, int *y);

	// Creates and initialize all necessary parameters to do
	// color tracking
	void  InitializeColorTracking (int index);

	// Gets a histogram stored in a file
	void GetHistFromFile (const char *file, int index);

	// Calculates backproject of image
	int CalculateColorBackground (char *file, int num_color, int maxx, int maxy, 
									int minx, int miny,	int vmin, int smin);

	// Calculates backproject of image
	//int CalculateColorBackground2 (char *file, int num_color);

	// Sets the threshold of the filter of cursor position
	void SetNoiseThreshold (int noise);

	// Creates an histogram contained in a square of 20x20
	// center by position x, y
	int CreateHistogram(int x, int y, int index);

	// Pass the information of histogram to an IplImage to can be showed
	IplImage* HistogramToIplImage(CvHistogram* hist, IplImage* histimg, int hdims); 

	// Transform from one space of color (hsv) to another (rgb)
	CvScalar hsv2rgb( float hue );

	// Save a histogram to an xml file
	void SaveHistFile (const char *file, int index);

	// draw board mode
	void WriteNumbers (int punts);

	// per dibuixar els punts en mode tauleta
	void WritePoints (int p1x, int p1y, int p2x, int p2y, int p3x,
                                int p3y, int p4x, int p4y);

	// Draw cercle centered to x, y
	void DrawCircle (int x, int y);

	// escriure linies en mode Joystick
	void PaintLines(int bh, int bv, int bh1, int bh2, int bv1, int bv2);

	// Save tuning parameters in a file
	int SaveParameters (char *file, int index, int maxx, int maxy, int minx, int miny,
										int vmin, int smin);

	int GetParameters (char *file, int index, int *maxx, int *maxy, int *minx, int *miny,
										int *vmin, int *smin);
	
	/****************************************************
	*
	*  MOVE TRACKING
	*
	*****************************************************/

	// Returns the number of white pixels over the total area
	// in per cent
	int	  GetPixelsNoZero (int x, int y, int w, int h);

	// Creates and initialize all necessary parameters to do
	// move tracking
	int  InitializeMoveTracking ();

	// Compute the zones of image with movement
	int  CalculateMovement();

	// Change the value defined for the move threshold to reduce
	// noise. Default = 100	
	int SetMoveThreshold (int thres);



private:

	// DEBUG
	//FILE *fp;


	/****************************************************
	*  GENERAL
	*****************************************************/
	CvCapture		*capture;					// Video capture structure
	IplImage		*frame;						// Image captured
	IplImage		*image;						// Copy of captured image
	int				xsize;						// Width of captured image
	int				ysize;						// Height of captured image
	int				bppixel;					// Bytes per pixel
	int				mode;						// Initializes for color/move or pattern tracking
	int				f_vert;						// Flips image vertical
	int				f_horz;						// Flips image horizontal

	/****************************************************
	*  COLOR TRACKING
	*****************************************************/
	IplImage		*imagecp[MAX_COLORS];		// Copy of image
	IplImage		*hsv;						// Image with hsv color transform
	IplImage		*hue;						// Image with hue color transform
	IplImage		*mask[MAX_COLORS];			// To get colors
	IplImage		*cbackprj[MAX_COLORS];		// Image projection (black&white)
	IplImage		*cbackprj4[MAX_COLORS];
	IplImage		*bprjcp;					// Copy of original backproject
	IplImage		*histgen;					// Image for the image histogram
	IplImage		*histimg[MAX_COLORS];		// Image for the rectangle histogram
	CvHistogram		*hist[MAX_COLORS];			// Rectangle histogram
	CvHistogram		*hist_foto;					// Image histogram
	int				distmin[MAX_COLORS];		// Minimum distance
//	CvPoint			origen;						// Origin coordenades
//	CvPoint			ptmp;						// Temporal point
	int				soroll;						// threshold to filter noise
	CvMemStorage	*cstorage[MAX_COLORS];		// Temporary storage
	int				find[MAX_COLORS];			// Indicates if color is found
	int				puntx[MAX_COLORS];			// Indicate position of color
	int				punty[MAX_COLORS];

	/****************************************************
	*  MOVE TRACKING
	*****************************************************/
	IplImage		*mbackprj;					// Image of movement (background)
	IplImage		*grey;						// For calculate difference
	IplImage		*add_abs_image;				// between two images
	IplImage		*pre_image;					// 
	IplImage		*pre_grey;					// 
	IplImage		*abs_image;					// 
	IplImage		*abs_images;				// 
	IplImage		*mbackprj4;					// 3-channel image (background)
	int				punts_no_zero;				// Non zero points
	CvMemStorage	*mstorage;					// Temporary storage
	CvSeq			*contour;					// Markers detected
	int				mthreshold;					// Move threshold to reduce noise

};


#endif
