/*
  CSprite.h
  Simple SDL sprite class.
  Made by Marius Andra 2001
  http://cone3d.gamedev.net

  You can use the code for anything you like.
  Even in a commercial project.
  But please let me know where it ends up.
  I'm just curious. That's all.

  Modified to allow draw alpha bitmaps
*/
#include <string.h>
#include "CSprite.h"

CSprite::CSprite():mFrame(0)
{
}

int CSprite::init(SDL_Surface *screen, char *dir)
{
	char buffer[255];
	char filename[255];
	char name[255];
	char strso[255];
	int pause=0, r=0, g=0, b=0;
	FILE *fp;
	mH = 0;
	mW = 0;
	mX = 0;
	mY = 0;

	sprintf(filename, "%s/info", dir);

	if((fp=fopen(filename, "r")) == NULL)
	{
		printf("ERROR opening file %s\n\n", filename);
		return -1;
	}

	fgets(buffer, 255, fp);  
	sscanf(buffer, "FILES: %d", &mNumframes);  
	mAnim = new CSpriteFrame[mNumframes];

	so = new CSound[mNumframes];

	mBuilt = 1;
	int count = 0;

	while(!feof(fp) && count<mNumframes)
	{
		fgets(buffer, 255, fp);
		if(buffer[0] != '#' && buffer[0] != '\r' && buffer[0] != '\0' && buffer[0] != '\n' && strlen(buffer) != 0)
		{
			sscanf(buffer, "%s %d %d %d %d %s", name, &pause, &r, &g, &b, strso);
			sprintf(filename, "%s/%s", dir, name);
			SDL_Surface *temp;
			//if((temp = SDL_LoadBMP(filename)) == NULL) return -1;
			if((temp = IMG_Load(filename)) == NULL) return -1;
			if(r >= 0) SDL_SetColorKey(temp, SDL_SRCCOLORKEY, SDL_MapRGB(temp->format, r, g, b));
			mAnim[count].image = SDL_DisplayFormat(temp);
			SDL_FreeSurface(temp);

			// load sound
			if ((strcmp(strso, "*") < 0) || (strcmp(strso, "*") > 0)){
				int ret = so[count].LoadEffect (strso);
				printf("ret %d\n", ret);
			}
			else so[count].NoLoadEffect ();

			mAnim[count].pause = pause;
			if(!mW) mW = mAnim[count].image->w; 
			if(!mH) mH = mAnim[count].image->h; //w

			count++;
		}
	}
	fclose(fp);	

	if(mBuilt)
	{
		if(mNumframes>1) mAnimating=1;
		mBackreplacement = SDL_DisplayFormat(mAnim[0].image);
	}
	mScreen = screen;
	
	return 0;
}

void CSprite::playSound (int frame, int repeat)
{
	so[frame].PlayEffect(repeat);
}

void CSprite::clearBG()
{
  if(mDrawn==1)
  {
    SDL_Rect dest;
    dest.x = (int)mOldX;
    dest.y = (int)mOldY;
    dest.w = mW;
    dest.h = mH;
    SDL_BlitSurface(mBackreplacement, NULL, mScreen, &dest);
  }
}

void CSprite::updateBG()
{
  SDL_Rect srcrect;
  srcrect.x = (int)mX;
  srcrect.y = (int)mY;
  srcrect.w = mW;
  srcrect.h = mH;
  mOldX=mX;mOldY=mY;
  SDL_BlitSurface(mScreen, &srcrect, mBackreplacement, NULL);
}

void CSprite::draw()
{
  if(mAnimating == 1)
  {
    if(mLastupdate+mAnim[mFrame].pause*mSpeed<SDL_GetTicks())
    {
      mFrame++;
      if(mFrame>mNumframes-1) mFrame=0;
      mLastupdate = SDL_GetTicks();
    }
  }

  if(mDrawn==0) mDrawn=1;

  SDL_Rect dest;
  dest.x = (int)mX; dest.y = (int)mY;
  SDL_BlitSurface(mAnim[mFrame].image, NULL, mScreen, &dest);
}

void CSprite::drawA(int alpha)
{
  if(mAnimating == 1)
  {
    if(mLastupdate+mAnim[mFrame].pause*mSpeed<SDL_GetTicks())
    {
      mFrame++;
      if(mFrame>mNumframes-1) mFrame=0;
      mLastupdate = SDL_GetTicks();
    }
  }

  if(mDrawn==0) mDrawn=1;

  SDL_SetAlpha(mAnim[mFrame].image, SDL_SRCALPHA, alpha);
  SDL_Rect dest;
  dest.x = (int)mX; dest.y = (int)mY;
  SDL_BlitSurface(mAnim[mFrame].image, NULL, mScreen, &dest);
}

int CSprite::esDins (int x, int y)
{
	if ((mX < x) && (x < mX+mW) && (mY < y) && ( y < mY+mH )) return 1;
	else return 0;
}

SDL_Surface* CSprite::ImageLoad(char *file)
{
	SDL_Surface *temp1, *temp2;
	temp1 = IMG_Load(file);
	temp2 = SDL_DisplayFormat(temp1);
	SDL_FreeSurface(temp1);
	return temp2;
}

void CSprite::DrawIMG(SDL_Surface *img, int x, int y, SDL_Surface *screen)
{
	SDL_Rect dest;
	dest.x = x;
	dest.y = y;
	SDL_BlitSurface(img, NULL, screen, &dest);
}

// Blit a portion of an image on the screen surface
void CSprite::DrawIMG(SDL_Surface *img, int x, int y,
                        int w, int h, int x2, int y2, SDL_Surface *screen)
{
  SDL_Rect dest;
  dest.x = x;
  dest.y = y;
  SDL_Rect dest2;
  dest2.x = x2;
  dest2.y = y2;
  dest2.w = w;
  dest2.h = h;
  SDL_BlitSurface(img, &dest2, screen, &dest);
}

SDL_Surface* CSprite::FrameLoad (void *pixels)
{
	SDL_Surface *temp1;
	SDL_Surface *temp2;
    Uint32 rmask, gmask, bmask, amask;

#if SDL_BYTEORDER == SDL_BIG_ENDIAN
    rmask = 0xff000000;
    gmask = 0x00ff0000;
    bmask = 0x0000ff00;
    amask = 0x000000ff;
#else
    rmask = 0x000000ff;
    gmask = 0x0000ff00;
    bmask = 0x00ff0000;
    amask = 0xff000000;
#endif

	temp1 = SDL_CreateRGBSurfaceFrom(pixels, 640, 480, 24, 1920, 
                                      rmask, gmask, bmask, amask);

	temp2 = SDL_DisplayFormat(temp1);
	SDL_FreeSurface(temp1);
	return temp2;
}
